/* ========================================
 *
 * Copyright AP Engineering, 2018
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your AP Engineering.
 *
 * ========================================
*/

#include "UART_commands.h"
#include "Error_Debug.h"

#include "FreeRTOSConfig.h"
#include "FreeRTOS.h" 
#include "task.h" 
#include "queue.h"

#include <stdio.h>
#include <string.h>

#if defined(__cplusplus)
extern "C" {
#endif

/***************************************
*            Constants
****************************************/
#define USER_RESPONSE_SIZE 100

/*******************************************************************************
*        Global Variables
*******************************************************************************/
uint32_t UART_command;
char CharacterData;
uint32_t DataReceived;
char UserResponse[USER_RESPONSE_SIZE];
uint8_t UART_error;


/*******************************************************************************
*        Local Prototypes
*******************************************************************************/
static void handleCommands();
static void getUserReponse();
static void handleUART_error();

static void reportTubeSensorsCommand();

/*******************************************************************************
*        Global Functions
*******************************************************************************/

/***************************************************************************//**
* Function Name: init_UART
********************************************************************************
*
* Summary:
*  This function initializes the UART used for Diagnostic communication.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  None
*
********************************************************************************/  
void init_UART()
{
    cy_en_scb_uart_status_t init_status;
    
    // Start UART operation.
    init_status = Cy_SCB_UART_Init( UART_HW, &UART_config, &UART_context );
    if( init_status != CY_SCB_UART_SUCCESS )
    {
        handleUART_error();
    }
    Cy_SCB_UART_Enable( UART_HW ); 
    
    // Unmasking only the RX fifo not empty interrupt bit
    UART_HW->INTR_RX_MASK = SCB_INTR_RX_MASK_NOT_EMPTY_Msk;
    
    Cy_SysInt_Init( &UART_SCB_IRQ_cfg, ISR_UART );
}

/***************************************************************************//**
* Function Name: ISR_UART
********************************************************************************
*
* Summary:
*  This function is registered to be called when UART interrupt occurs.
*  (Note that only RX fifo not empty interrupt is unmasked)
*  Whenever there is a data in UART RX fifo, Get that data and Put it into 
*  UART TX fifo which will be transmitted to terminal
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  None
*
*******************************************************************************/
void ISR_UART(void)
{
    char str[3];
    
    /* Check for "RX fifo not empty interrupt" */
    if (( UART_HW->INTR_RX_MASKED & SCB_INTR_RX_MASKED_NOT_EMPTY_Msk ) != 0 )
	{
        /* Get the character from terminal */
		CharacterData = Cy_SCB_UART_Get( UART_HW );
        
        /* Clear UART "RX fifo not empty interrupt" */
		UART_HW->INTR_RX = UART_HW->INTR_RX & SCB_INTR_RX_NOT_EMPTY_Msk;
        
        // Valid ASCII
        if ( 
                ( 32 <= CharacterData && 126 >= CharacterData ) ||
                (uint32_t)('\n') == CharacterData
            )
        {        
            // echo
            sprintf( str, "%s", &CharacterData ); 
            Cy_SCB_UART_PutString( UART_HW, str );
        } 
        
        /* Update data_received flag */
        DataReceived = 1;   	
    }   
    else
    {
        /* Error if any other interrupt occurs */
        UART_error = 1;
    }
}

/*******************************************************************************
* Function Name: displayCommandMenu
********************************************************************************
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void displayCommandMenu()
{
    /* \x1b[2J\x1b[;H - ANSI ESC sequence for clear screen */
    Cy_SCB_UART_PutString( UART_HW, "\033[;H\033[2J\033[;H" );
    
    Cy_SCB_UART_PutString( UART_HW, "\r\n**********************************\r\n" );
    Cy_SCB_UART_PutString( UART_HW, "Commands: \r\n\r\n" );

    Cy_SCB_UART_PutString( UART_HW, "r) Report Tube Sensor readings.\r\n" );

    Cy_SCB_UART_PutString( UART_HW, "\r\n   command > " );
}


/***************************************************************************//**
* Function Name: UART_MenuTask
********************************************************************************
*
* Summary:
*   This is the FreeRTOS task for the UART communication.

* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  None
*
*******************************************************************************/
void UART_MenuTask( CY_UNUSED void *arg )
{
    for(;;)
    {
        vTaskDelay(1);
        
        /* Wait for RX fifo not empty interrupt */
        /* Handle received data from terminal */
        if( DataReceived == 1 )
        {
            DataReceived = 0;

            if ( (uint32_t)('\n') == CharacterData )
            {
                handleCommands();
            }
            else
            {
                if ( 32 <= CharacterData && 126 >= CharacterData )
                {
                    UART_command = CharacterData;
                }
            }
        }
        
        /* Handle UART error */
        if( UART_error == 1 )
        {
            handleUART_error();
        }
    } 
}

/*******************************************************************************
*            Local Functions
*******************************************************************************/
static void getUserReponse()
{
     /* Wait for RX fifo not empty interrupt */
    int n = 0;
    memset( UserResponse, 0, USER_RESPONSE_SIZE );
    
    for(;;)
    {          
        taskYIELD();
            
        /* Handle received data from terminal */
        if( DataReceived == 1 )
        {
            DataReceived = 0;

            if ( ( uint32_t )( '\n') == CharacterData )
            {
                UserResponse[n++] = 0;
                return;
            }
            else
            {
                if ( 32 <= CharacterData && 126 >= CharacterData )
                {
                    if ( n < USER_RESPONSE_SIZE )
                    {
                        UserResponse[n++] = CharacterData;
                    }
                }
            }
        }
    }

    /* Handle UART error */
    if ( UART_error == 1 )
    {
        handleUART_error();
    }
}

static void waitForUser()
{
    Cy_SCB_UART_PutString( UART_HW, "\r\n (press any key to continue)" );
    while ( DataReceived == 0 ) { vTaskDelay( 10 ); }
    DataReceived = 0;
}

static void handleCommands()
{
    switch ( UART_command )
    {
        case CMD_REPORT_TUBE_SENSOR:
            reportTubeSensorsCommand();
            break;           
         
        default:
            Cy_SCB_UART_PutString( UART_HW, "\r\n\tUnknown command.\r\n" );
            waitForUser();
            PumpSensor_timeout_TriggerReload();
            PumpSensor_timeout_TriggerStart(); 
    }
    displayCommandMenu();
}

static void handleUART_error()
{   
    REPORT_ERROR()
    handle_error();
}

static void reportTubeSensorsCommand()
{
     
    waitForUser();
}

#if defined(__cplusplus)
}
#endif

/* [] END OF FILE */
